{# vim: set ts=2 et sts=2 sw=2: #}
{% extends "tidings/base.html" %}

{% load i18n %}

{% block head_title %}{% trans "Unsubscribe" %}{% endblock %}

{% block content %}
  <h1>{% trans "Unsubscribe Error" %}</h1>
  <p>
    {% blocktrans trimmed %}
      We could not find your subscription. Either it has already been
      cancelled, or there was a mistake in the unsubscribe link. Please make
      sure the entire link from the email made it into the browser. If the last
      part of the link wraps onto a second line in the email, try copying and
      pasting the link into the browser.
    {% endblocktrans %}
  </p>
</article>
{% endblock %}
