{# vim: set ts=2 et sts=2 sw=2: #}
{% extends "tidings/base.html" %}
{% load i18n %}

{# Having this change to "Unsubscribed" draws my eye and bothers me. #}
{% block head_title %}{% trans "Unsubscribe" %}{% endblock %}

{% block content %}
<h1>{% trans "Unsubscribed" %}</h1>

<p>{% trans "You have been unsubscribed." %}</p>
{% endblock %}
