{# vim: set ts=2 et sts=2 sw=2: #}
{% extends "tidings/base.html" %}
{% load i18n %}

{% block head_title %}{% trans "Unsubscribe" %}{% endblock %}

{% block content %}
  <h1>{% trans "Are you sure you want to unsubscribe?" %}</h1>
  {# TODO: Once Watches know how to describe themselves, print that description. #}
  <form action="" method="post">
    {% csrf_token %}
    <div class="submit-or-cancel">
      <input type="submit" value="{% trans "Unsubscribe" %}" />
      <a href="/"{# Discards l10n. Better ideas? #}>{% trans "Cancel" %}</a>
    </div>
  </form>
{% endblock %}
